//
//  ConcentrationTest.swift
//  ConcentrationTest
//
//  Created by Luciano Bononi on 12/03/2020.
//  Copyright © 2020 Luciano Bononi. All rights reserved.
//

import Foundation

class ConcentrationTest {
    private(set) var cards = [Card]()
    
// computer property: ritorna tramite getter l'indice della sola card a faccia in su, se esiste, altrimenti nil
   private var indexOfOneAndOnlyFaceUpCard: Int? {
        get {
            var foundIndex: Int?
            for index in cards.indices {
                if cards[index].isFaceUp {
                    if foundIndex == nil {
                        foundIndex = index
                    } else {
                        return nil
                    }
                }
            }
            return foundIndex
        }
    // computed property: questo è il setter dell var che contiene l'indice della sola card a faccia in su. Notare che setta nel modello la card a faccia in su, mettendo a faccia in giu tutte le altre.
        set(newValue) {
            for index in cards.indices {
                cards[index].isFaceUp = (index == newValue)
            }
        }
    }
    
    func chooseCard(at index: Int) {
        // assert: se il predicato non è vero scrive in console
        assert(cards.indices.contains(index), "Concentration.chooseCard(at: \(index)): chosen index is not in the cards")
        // se la card[index] non è matched
        if !cards[index].isMatched {
            // se matchIndex è la sola card a faccia in su e ha indice diverso da index, che è la seconda card girata
            if let matchIndex = indexOfOneAndOnlyFaceUpCard, matchIndex != index {
                //check if cards match. NB si guardano gli identifier
                if cards[matchIndex].identifier == cards[index].identifier {
                    cards[matchIndex].isMatched = true
                    cards[index].isMatched = true
                }
                cards[index].isFaceUp = true
                //               indexOfOneAndOnlyFaceUpCard = nil //NB set to nil since it is defined as Optional, but not needed after computed property was created
            } else {
                // either no cards or 2 cards are face Up, first put all cards face down
                //                for flipDownIndex in cards.indices {
                //                    cards[flipDownIndex].isFaceUp = false
                //                }
                //                cards[index].isFaceUp = true
                indexOfOneAndOnlyFaceUpCard = index
            }
        }
    }
    
    func reset() {
        // alternative function for resetting the game, to do as homework
    }
    
    init(numberOfPairsOfCards: Int) {
        assert(numberOfPairsOfCards > 0, "Concentration.init(\(numberOfPairsOfCards)): you must have at least one pair of cards")
        for _ in 1...numberOfPairsOfCards {
        let card = Card()
        cards += [card, card]
        }
        
        // TODO: shuffle cards
         for swapFromIndex in stride(from: 2, through: 2*(numberOfPairsOfCards - 1), by: 2) {
            let swapToIndex = Int(arc4random_uniform(UInt32(numberOfPairsOfCards))) * 2
            let temp = cards[swapFromIndex].identifier
            cards[swapFromIndex].identifier = cards[swapToIndex].identifier
            cards[swapToIndex].identifier = temp
         }
    }
}
